import { defineComponent, PropType, ref, watch } from 'vue';
import getSlot from '@/utils/getSolt';
import './index.scss';
export interface Schema {
  type: 'el-date-picker' | 'el-input' | 'el-select';
  defaultValue?: unknown;
  key: string;
  options?: {
    value: string | number | boolean | object;
    label?: string | number;
    disabled?: boolean;
  }[];
  placeholder?: string;
  label?: string;
}

export interface QueryFormEvents {
  onQuery: () => void;
}

export type FormModule<T extends object> = {
  [key in keyof T]: T[keyof T];
};

export default defineComponent({
  name: 'QueryForm',
  props: {
    schema: Array as PropType<Schema[]>,
    onSubmit: Function,
    onReset: Function,
    form: { type: Object, required: true },
    suffix: {
      type: [Object, String] as PropType<JSX.Element | string>,
      default: undefined,
    },
  },
  setup(props, ctx) {
    const suffix = getSlot({ props, ctx }, 'suffix');
    const form = ref(props.form);

    const handleSubmit = () => {
      props.onSubmit && props.onSubmit(form.value);
    };

    if (!props.schema) {
      return null;
    }

    watch(
      () => props.form,
      (next) => {
        form.value = next;
      },
    );

    return () => (
      <div class="query-inline-form-container">
        <el-form inline class="form-inline">
          {props.schema &&
            props.schema.map((item) => {
              return (
                <>
                  {item.type === 'el-input' && (
                    <el-form-item label={item.label}>
                      <el-input
                        size="small"
                        v-model={form.value[item.key]}
                        placeholder={item.placeholder || '请输入'}
                        clearable
                      />
                    </el-form-item>
                  )}
                  {item.type === 'el-select' && (
                    <el-form-item label={item.label}>
                      <el-select
                        clearable
                        size="small"
                        v-model={form.value[item.key]}
                        placeholder={item.placeholder || '请选择'}>
                        {item.options &&
                          item.options.map((o) => (
                            <el-option
                              label={o.label}
                              value={o.value}
                              disabled={o.disabled}
                            />
                          ))}
                      </el-select>
                    </el-form-item>
                  )}
                  {item.type === 'el-date-picker' && (
                    <el-form-item label={item.label}>
                      <el-date-picker
                        size="small"
                        v-model={form.value[item.key]}
                        style={{ width: '200px' }}
                        type="daterange"
                        start-placeholder="开始时间"
                        end-placeholder="结束时间"
                        format="YYYY-MM-DD"
                        valueFormat="YYYY-MM-DD"
                      />
                    </el-form-item>
                  )}
                </>
              );
            })}
          <el-form-item>
            <el-button type="primary" size="small" onClick={handleSubmit}>
              查询
            </el-button>
          </el-form-item>
          <el-form-item>{suffix}</el-form-item>
        </el-form>
      </div>
    );
  },
});
